@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2001-2016 Andrew Apted
+//  Copyright (C) 2001-2018 Andrew Apted
 //  Copyright (C) 1997-2003 Andr Majorel et al
 //
 //  This program is free software; you can redistribute it and/or
@@ -763,6 +763,34 @@ static const opt_desc_t options[] =
 
 //------------------------------------------------------------------------
 
+// this automatically strips CR/LF from the line.
+// returns true if ok, false on EOF or error.
+bool M_ReadTextLine(char *buf, size_t size, FILE *fp)
+{
+	if (! fgets(buf, static_cast<int>(size), fp))
+	{
+		buf[0] = 0;
+		return false;
+	}
+
+	// remove a Unicode BOM (byte-order mark)
+	if ((byte)buf[0] == 0xEF &&
+		(byte)buf[1] == 0xBB &&
+		(byte)buf[2] == 0xBF)
+	{
+		size_t len = strlen(buf) - 3;
+
+		memmove(buf, buf+3, len);
+
+		buf[len] = 0;
+	}
+
+	StringRemoveCRLF(buf);
+
+	return true;
+}
+
+
 static int parse_config_line_from_file(char *p, const char *basename, int lnum)
 {
 	char *name  = NULL;
@@ -912,8 +940,8 @@ static int parse_a_config_file(FILE *fp, const char *filename)
 
 	const char *basename = FindBaseName(filename);
 
-	// Execute one line on each iteration
-	for (int lnum = 1 ; fgets(line, sizeof(line), fp) != NULL ; lnum++)
+	// handle one line on each iteration
+	for (int lnum = 1 ; M_ReadTextLine(line, sizeof(line), fp) ; lnum++)
 	{
 		int ret = parse_config_line_from_file(line, basename, lnum);
 
@@ -1457,19 +1485,12 @@ bool M_LoadUserState()
 	LogPrintf("Loading user state from: %s\n", filename);
 
 
-	static char line_buf[FL_PATH_MAX];
+	static char line[FL_PATH_MAX];
 
 	const char * tokens[MAX_TOKENS];
 
-	while (! feof(fp))
+	while (M_ReadTextLine(line, sizeof(line), fp))
 	{
-		char *line = fgets(line_buf, FL_PATH_MAX, fp);
-
-		if (! line)
-			break;
-
-		StringRemoveCRLF(line);
-
 		int num_tok = M_ParseLine(line, tokens, MAX_TOKENS, true /* do_strings */);
 
 		if (num_tok == 0)
@@ -1543,6 +1564,8 @@ void M_DefaultUserState()
 	ZoomWholeMap();
 
 	Render3D_Setup();
+
+	Editor_DefaultState();
 }
 
 
