@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2001-2016 Andrew Apted
+//  Copyright (C) 2001-2018 Andrew Apted
 //  Copyright (C) 1997-2003 Andr Majorel et al
 //
 //  This program is free software; you can redistribute it and/or
@@ -220,7 +220,7 @@ public:
 
 	void SaveOffsets()
 	{
-		unsigned int total = adjust_sides.size();
+		unsigned int total = static_cast<unsigned>(adjust_sides.size());
 
 		if (total == 0)
 			return;
@@ -246,7 +246,7 @@ public:
 
 	void RestoreOffsets()
 	{
-		unsigned int total = adjust_sides.size();
+		unsigned int total = static_cast<unsigned>(adjust_sides.size());
 
 		for (unsigned int k = 0 ; k < total ; k++)
 		{
@@ -619,9 +619,71 @@ public:
 	DrawSurf floor;
 	DrawSurf rail;
 
-#define IZ_EPSILON  1e-6
+	// IsCloser tests if THIS wall (wall A) is closer to the camera
+	// than the given wall (wall B).
+	//
+	// Note that it is NOT suitable as a predicate for std::sort()
+	// since it does not guarantee a linear order (total order) of
+	// the elements.  Hence the need for our own sorting code.
 
-   /* PREDICATES */
+	inline bool IsCloser(const DrawWall *const B) const
+	{
+		const DrawWall *const A = this;
+
+		if (A == B)
+			return false;
+
+		if (A->ld && B->ld)
+		{
+			// handle cases where two linedefs share a vertex, since that
+			// is where slime-trails would otherwise occur.
+
+			// if they do share a vertex, we check if the other vertex of
+			// wall A and the camera position are both on the same side of
+			// wall B (extended to infinity).
+
+			int A_other = -1;
+
+			if (B->ld->TouchesVertex(A->ld->start))
+				A_other = A->ld->end;
+			else if (B->ld->TouchesVertex(A->ld->end))
+				A_other = A->ld->start;
+
+			if (A_other >= 0)
+			{
+				// TODO : optimize this (if possible)
+
+				int ax = Vertices[A_other]->x;
+				int ay = Vertices[A_other]->y;
+
+				int bx1 = B->ld->Start()->x;
+				int by1 = B->ld->Start()->y;
+				int bx2 = B->ld->End()->x;
+				int by2 = B->ld->End()->y;
+
+				int cx = (int)view.x;  // camera
+				int cy = (int)view.y;
+
+				int A_side = PointOnLineSide(ax, ay, bx1, by1, bx2, by2);
+				int C_side = PointOnLineSide(cx, cy, bx1, by1, bx2, by2);
+
+				return (A_side * C_side >= 0);
+			}
+		}
+		else if (A->th >= 0 && B->th >= 0)
+		{
+			// prevent two things at same location from flickering
+			const Thing *const TA = Things[A->th];
+			const Thing *const TB = Things[B->th];
+
+			if (TA->x == TB->x && TA->y == TB->y)
+				return A->th > B->th;
+		}
+
+		return A->cur_iz > B->cur_iz;
+	}
+
+	/* PREDICATES */
 
 	struct MidDistCmp
 	{
@@ -631,55 +693,6 @@ public:
 		}
 	};
 
-	struct DistCmp
-	{
-		inline bool operator() (const DrawWall * A, const DrawWall * B) const
-		{
-			if (fabs(A->cur_iz - B->cur_iz) >= IZ_EPSILON)
-			{
-				// this is the normal case
-				return A->cur_iz > B->cur_iz;
-			}
-
-			// this case usually occurs at a column where two walls share a vertex.
-			//
-			// hence we check if they actually share a vertex, and if so then
-			// we test whether A is behind B or not -- by checking which side
-			// of B the camera and the other vertex of A are on.
-
-			if (A->ld && B->ld)
-			{
-				// find the vertex of A _not_ shared with B
-				int A_other = -1;
-
-				if (B->ld->TouchesVertex(A->ld->start)) A_other = A->ld->end;
-				if (B->ld->TouchesVertex(A->ld->end))   A_other = A->ld->start;
-
-				if (A_other >= 0)
-				{
-					int ax = Vertices[A_other]->x;
-					int ay = Vertices[A_other]->y;
-
-					int bx1 = B->ld->Start()->x;
-					int by1 = B->ld->Start()->y;
-					int bx2 = B->ld->End()->x;
-					int by2 = B->ld->End()->y;
-
-					int cx = (int)view.x;  // camera
-					int cy = (int)view.y;
-
-					int A_side = PointOnLineSide(ax, ay, bx1, by1, bx2, by2);
-					int C_side = PointOnLineSide(cx, cy, bx1, by1, bx2, by2);
-
-					return (A_side * C_side >= 0);
-				}
-			}
-
-			// a pretty good fallback:
-			return A->mid_iz > B->mid_iz;
-		}
-	};
-
 	struct SX1Cmp
 	{
 		inline bool operator() (const DrawWall * A, const DrawWall * B) const
@@ -880,6 +893,7 @@ public:
 	// inverse distances over X range, 0 when empty.
 	std::vector<double> depth_x;
 
+	// vertical clip window, an inclusive range
 	int open_y1;
 	int open_y2;
 
@@ -1633,24 +1647,22 @@ public:
 		int y1 = DistToY(dw->cur_iz, surf.h2);
 		int y2 = DistToY(dw->cur_iz, surf.h1) - 1;
 
+		// clip to the open region
 		if (y1 < open_y1)
 			y1 = open_y1;
 
 		if (y2 > open_y2)
 			y2 = open_y2;
 
-		if (y1 > y2)
-			return;
-
-		/* clip the open region */
-
+		// update open region based on ends which are "solid"
 		if (surf.y_clip & DrawSurf::SOLID_ABOVE)
-			if (open_y1 < y2)
-				open_y1 = y2;
+			open_y1 = MAX(open_y1, y2 + 1);
 
 		if (surf.y_clip & DrawSurf::SOLID_BELOW)
-			if (open_y2 > y1)
-				open_y2 = y1;
+			open_y2 = MIN(open_y2, y1 - 1);
+
+		if (y1 > y2)
+			return;
 
 		/* query mode : is mouse over this wall part? */
 
@@ -1792,6 +1804,139 @@ public:
 		RenderTexColumn(dw, surf, x, y1, y2);
 	}
 
+	inline void Sort_Swap(int i, int k)
+	{
+		DrawWall *A = active[i];
+		DrawWall *B = active[k];
+
+		active[k] = A;
+		active[i] = B;
+	}
+
+	int Sort_Partition(int lo, int hi, int pivot_idx)
+	{
+		/* this is Hoare's algorithm */
+
+		const DrawWall *pivot = active[pivot_idx];
+
+		int s = lo;
+		int e = hi;
+
+		for (;;)
+		{
+			while (s <= e && active[s]->IsCloser(pivot))
+				s++;
+
+			if (s > hi)
+			{
+				// all values were < pivot, including the pivot itself!
+
+				if (pivot_idx != hi)
+					Sort_Swap(pivot_idx, hi);
+
+				return hi - 1;
+			}
+
+			while (e >= s && ! active[e]->IsCloser(pivot))
+				e--;
+
+			if (e < lo)
+			{
+				// all values were >= pivot
+
+				if (pivot_idx != lo)
+					Sort_Swap(pivot_idx, lo);
+
+				return lo;
+			}
+
+			if (s < e)
+			{
+				Sort_Swap(s, e);
+
+				s++;
+				e--;
+
+				continue;
+			}
+
+			/* NOT NEEDED (it seems)
+			if (s == e && active[s]->IsCloser(pivot))
+				s++;
+			*/
+
+			return s - 1;
+		}
+	}
+
+	void Sort_Range(int s, int e)
+	{
+		SYS_ASSERT(s <= e);
+
+		while (s < e)
+		{
+			// previously there was a bubble sort here, but timing
+			// tests showed that it was overkill.  This is enough.
+			if (s == e-1)
+			{
+				const DrawWall *const A = active[s];
+				const DrawWall *const B = active[e];
+
+				if (B->IsCloser(A))
+					Sort_Swap(s, e);
+
+				return;
+			}
+
+			// since we are usually sorting a mostly-sorted list, the
+			// wall in the middle is highly likely to be a good pivot.
+			int pivot_idx = (s + e) >> 1;
+
+			int mid = Sort_Partition(s, e, pivot_idx);
+
+			// handle degenerate cases
+			if (mid <= s)
+			{
+				s++;
+				continue;
+			}
+			else if (mid+1 >= e)
+			{
+				e--;
+				continue;
+			}
+
+			// only use recursion on the smallest group
+			// [ it helps to limit stack usage ]
+			if ((mid - s) < (e - mid))
+			{
+				Sort_Range(s, mid);
+				s = mid+1;
+			}
+			else
+			{
+				Sort_Range(mid+1, e);
+				e = mid;
+			}
+		}
+	}
+
+	void SortActiveList()
+	{
+		// this uses the Quicksort algorithm to sort the active list.
+		//
+		// Note that this sorting code has been written assuming some
+		// limitations of the DrawWall::IsCloser() method -- see the
+		// description of that method for more details.
+
+		if (active.size() < 2)
+			return;
+
+		Sort_Range(0, (int)active.size() - 1);
+	}
+
+#define IZ_EPSILON  1e-5
+
 	void UpdateActiveList(int x)
 	{
 		DrawWall::vec_t::iterator S, E, P;
@@ -1844,9 +1989,9 @@ public:
 
 		// if there are changes, re-sort the active list...
 
-		if (changes)
+		if (changes && active.size() > 0)
 		{
-			std::sort(active.begin(), active.end(), DrawWall::DistCmp());
+			SortActiveList();
 		}
 	}
 
@@ -1892,10 +2037,11 @@ public:
 
 				RenderWallSurface(dw, dw->ceil,  x, OB3D_Ceil);
 				RenderWallSurface(dw, dw->floor, x, OB3D_Floor);
+
 				RenderWallSurface(dw, dw->upper, x, OB3D_Upper);
 				RenderWallSurface(dw, dw->lower, x, OB3D_Lower);
 
-				if (open_y1 >= open_y2)
+				if (open_y1 > open_y2)
 					break;
 			}
 
@@ -2830,12 +2976,12 @@ static void Render3D_Delete()
 }
 
 
-bool Render3D_ClipboardOp(char what)
+bool Render3D_ClipboardOp(char op)
 {
 	if (r_edit.SelectEmpty() && ! r_edit.hl.valid())
 		return false;
 
-	switch (what)
+	switch (op)
 	{
 		case 'c':
 			Render3D_Copy();
