@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2012-2016 Andrew Apted
+//  Copyright (C) 2012-2018 Andrew Apted
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -664,6 +664,7 @@ public:
 	Fl_Check_Button *edit_newislands;
 	Fl_Check_Button *edit_samemode;
 	Fl_Check_Button *edit_autoadjustX;
+	Fl_Check_Button *edit_add_del;
 	Fl_Int_Input    *edit_sectorsize;
 
 	Fl_Check_Button *brow_smalltex;
@@ -678,6 +679,8 @@ public:
 	Fl_Choice *grid_size;
 
 	Fl_Check_Button *grid_hide_free;
+	Fl_Check_Button *grid_flatrender;
+	Fl_Check_Button *grid_spriterend;
 
 	Fl_Button *dotty_axis;
 	Fl_Button *dotty_major;
@@ -892,6 +895,8 @@ UI_Preferences::UI_Preferences() :
 		}
 		{ edit_samemode = new Fl_Check_Button(50, 180, 270, 30, " same mode key will clear selection");
 		}
+		{ edit_add_del = new Fl_Check_Button(50, 210, 270, 30, " enable sidedef ADD / DEL buttons");
+		}
 		{ edit_sectorsize = new Fl_Int_Input(440, 120, 105, 25, "new sector size:");
 		}
 
@@ -920,14 +925,18 @@ UI_Preferences::UI_Preferences() :
 		}
 		{ grid_enabled = new Fl_Check_Button(50, 125, 95, 25, " default grid to ON");
 		}
-		{ grid_snap = new Fl_Check_Button(50, 160, 235, 25, " default SNAP mode");
+		{ grid_snap = new Fl_Check_Button(50, 155, 235, 25, " default SNAP mode");
+		}
+		{ grid_flatrender = new Fl_Check_Button(50, 185, 270, 25, " default sector rendering to ON");
+		}
+		{ grid_spriterend = new Fl_Check_Button(50, 215, 270, 25, " default sprite rendering to ON");
 		}
 		{ grid_size = new Fl_Choice(400, 90, 95, 25, "default grid size ");
 		  grid_size->add("1024|512|256|192|128|64|32|16|8|4|2");
 		}
 		{ gen_scrollbars = new Fl_Check_Button(277, 125, 245, 25, " enable scroll-bars for map view");
 		}
-		{ grid_hide_free = new Fl_Check_Button(277, 160, 245, 25, " hide grid in FREE mode");
+		{ grid_hide_free = new Fl_Check_Button(277, 155, 245, 25, " hide grid in FREE mode");
 		}
 
 		{ Fl_Box* o = new Fl_Box(25, 270, 355, 30, "Grid Colors");
@@ -1418,6 +1427,7 @@ void UI_Preferences::LoadValues()
 	edit_sectorsize->value(Int_TmpStr(new_sector_size));
 	edit_newislands->value(new_islands_are_void ? 1 : 0);
 	edit_samemode->value(same_mode_clears_selection ? 1 : 0);
+	edit_add_del->value(sidedef_add_del_buttons ? 1 : 0);
 	edit_autoadjustX->value(leave_offsets_alone ? 0 : 1);
 
 	brow_smalltex->value(browser_small_tex ? 1 : 0);
@@ -1435,6 +1445,8 @@ void UI_Preferences::LoadValues()
 	grid_snap->value(grid_default_snap ? 1 : 0);
 	grid_size->value(GridSizeToChoice(grid_default_size));
 	grid_hide_free ->value(grid_hide_in_free_mode ? 1 : 0);
+	grid_flatrender->value(sector_render_default ? 1 : 0);
+	grid_spriterend->value(thing_render_default ? 1 : 0);
 
 	gen_scrollbars ->value(map_scroll_bars ? 1 : 0);
 
@@ -1539,6 +1551,7 @@ void UI_Preferences::SaveValues()
 
 	new_islands_are_void = edit_newislands->value() ? true : false;
 	same_mode_clears_selection = edit_samemode->value() ? true : false;
+	sidedef_add_del_buttons = edit_add_del->value() ? true : false;
 	leave_offsets_alone = edit_autoadjustX->value() ? false : true;
 
 	// changing this requires re-populating the browser
@@ -1555,7 +1568,9 @@ void UI_Preferences::SaveValues()
 	grid_default_mode = grid_enabled->value();
 	grid_default_snap = grid_snap->value() ? true : false;
 	grid_default_size = atoi(grid_size->mvalue()->text);
-	grid_hide_in_free_mode  = grid_hide_free ->value() ? true : false;
+	grid_hide_in_free_mode = grid_hide_free ->value() ? true : false;
+	sector_render_default  = grid_flatrender->value() ? 1 : 0;
+	thing_render_default   = grid_spriterend->value() ? 1 : 0;
 
 	map_scroll_bars = gen_scrollbars ->value() ? true : false;
 
